@@ -60,6 +60,7 @@
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
 #include "MagickCore/token.h"
+#include "MagickCore/timer-private.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/utility-private.h"
 #include "MagickCore/xml-tree.h"
@@ -283,7 +284,11 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
 static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
 {
   char
-    policyname[MagickPathExtent];
+    policyname[MagickPathExtent],
+    *q;
+
+  ElementInfo
+    *p;
 
   PolicyDomain
     domain;
@@ -291,12 +296,6 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
   PolicyInfo
     *policy;
 
-  ElementInfo
-    *p;
-
-  char
-    *q;
-
   assert(exception != (ExceptionInfo *) NULL);
   if (IsPolicyCacheInstantiated(exception) == MagickFalse)
     return((PolicyInfo *) NULL);
@@ -1104,8 +1103,8 @@ MagickPrivate void PolicyComponentTerminus(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SetMagickSecurityPolicy() sets the ImageMagick security policy.  It returns
-%  MagickFalse if the policy is already set or if the policy does not parse.
+%  SetMagickSecurityPolicy() sets or restricts the ImageMagick security policy.
+%  It returns MagickFalse if the policy the policy does not parse.
 %
 %  The format of the SetMagickSecurityPolicy method is:
 %
@@ -1150,32 +1149,46 @@ static MagickBooleanType ValidateSecurityPolicy(const char *policy,
 MagickExport MagickBooleanType SetMagickSecurityPolicy(const char *policy,
   ExceptionInfo *exception)
 {
-  PolicyInfo
-    *p;
-
   MagickBooleanType
     status;
 
+  LinkedListInfo
+    *user_policies;
+
+  PolicyInfo
+    *p;
+
+  /*
+    Load user policies.
+  */
   assert(exception != (ExceptionInfo *) NULL);
   if (policy == (const char *) NULL)
     return(MagickFalse);
   if (ValidateSecurityPolicy(policy,PolicyFilename,exception) == MagickFalse)
     return(MagickFalse);
-  if (IsPolicyCacheInstantiated(exception) == MagickFalse)
-    return(MagickFalse);
-  LockSemaphoreInfo(policy_semaphore);
-  ResetLinkedListIterator(policy_cache);
-  p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  if ((p != (PolicyInfo *) NULL) && (p->domain != UndefinedPolicyDomain))
-    {
-      UnlockSemaphoreInfo(policy_semaphore);
-      return(MagickFalse);
-    }
-  UnlockSemaphoreInfo(policy_semaphore);
   status=LoadPolicyCache(policy_cache,policy,"[user-policy]",0,exception);
   if (status == MagickFalse)
-    return(MagickFalse);
-  return(ResourceComponentGenesis());
+    return(status);
+  /*
+    Synchronize user policies.
+  */
+  user_policies=NewLinkedList(0);
+  status=LoadPolicyCache(user_policies,policy,"[user-policy]",0,exception);
+  if (status == MagickFalse)
+    {
+      user_policies=DestroyLinkedList(user_policies,DestroyPolicyElement);
+      return(MagickFalse);
+    }
+  ResetLinkedListIterator(user_policies);
+  p=(PolicyInfo *) GetNextValueInLinkedList(user_policies);
+  while (p != (PolicyInfo *) NULL)
+  {
+    if ((p->name != (char *) NULL) && (p->value != (char *) NULL))
+      (void) SetMagickSecurityPolicyValue(p->domain,p->name,p->value,exception);
+    p=(PolicyInfo *) GetNextValueInLinkedList(user_policies);
+  }
+  user_policies=DestroyLinkedList(user_policies,DestroyPolicyElement);
+  return(status);
 }
 
 /*
@@ -1211,60 +1224,15 @@ MagickExport MagickBooleanType SetMagickSecurityPolicy(const char *policy,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-
-static MagickBooleanType SetPolicyValue(const PolicyDomain domain,
-  const char *name,const char *value)
-{
-  MagickBooleanType
-    status;
-
-  PolicyInfo
-    *p;
-
-  status=MagickTrue;
-  LockSemaphoreInfo(policy_semaphore);
-  ResetLinkedListIterator(policy_cache);
-  p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  while (p != (PolicyInfo *) NULL)
-  {
-    if ((p->domain == domain) && (LocaleCompare(name,p->name) == 0))
-      break;
-    p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  }
-  if (p != (PolicyInfo *) NULL)
-    {
-      if (p->value != (char *) NULL)
-        p->value=DestroyString(p->value);
-    }
-  else
-    {
-      p=(PolicyInfo *) AcquireCriticalMemory(sizeof(*p));
-      (void) memset(p,0,sizeof(*p));
-      p->exempt=MagickFalse;
-      p->signature=MagickCoreSignature;
-      p->domain=domain;
-      p->name=AcquirePolicyString(name,1);
-      status=AppendValueToLinkedList(policy_cache,p);
-    }
-  p->value=AcquirePolicyString(value,1);
-  UnlockSemaphoreInfo(policy_semaphore);
-  if (status == MagickFalse)
-    p=(PolicyInfo *) RelinquishMagickMemory(p);
-  return(status);
-}
-
 MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
   const PolicyDomain domain,const char *name,const char *value,
   ExceptionInfo *exception)
 {
-  char
-    *current_value;
-
   magick_unreferenced(exception);
   assert(exception != (ExceptionInfo *) NULL);
   if ((name == (const char *) NULL) || (value == (const char *) NULL))
     return(MagickFalse);
-  switch(domain)
+  switch (domain)
   {
     case CachePolicyDomain:
     {
@@ -1274,10 +1242,8 @@ MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
             return(MagickFalse);
           ResetCacheAnonymousMemory();
           ResetStreamAnonymousMemory();
-          return(SetPolicyValue(domain,name,value));
+          return(MagickTrue);
         }
-      if (LocaleCompare(name,"synchronize") == 0)
-        return(SetPolicyValue(domain,name,value));
       break;
     }
     case ResourcePolicyDomain:
@@ -1285,8 +1251,6 @@ MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
       ssize_t
         type;
 
-      if (LocaleCompare(name,"temporary-path") == 0)
-        return(SetPolicyValue(domain,name,value));
       type=ParseCommandOption(MagickResourceOptions,MagickFalse,name);
       if (type >= 0)
         {
@@ -1296,52 +1260,30 @@ MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
           limit=MagickResourceInfinity;
           if (LocaleCompare("unlimited",value) != 0)
             limit=StringToMagickSizeType(value,100.0);
+          if ((ResourceType) type == TimeResource)
+            limit=(MagickSizeType) ParseMagickTimeToLive(value);
           return(SetMagickResourceLimit((ResourceType) type,limit));
         }
       break;
     }
     case SystemPolicyDomain:
     {
-      if (LocaleCompare(name,"font") == 0)
-        return(SetPolicyValue(domain,name,value));
       if (LocaleCompare(name,"max-memory-request") == 0)
         {
-          current_value=GetPolicyValue("system:max-memory-request");
-          if ((current_value == (char *) NULL) ||
-              (StringToSizeType(value,100.0) < StringToSizeType(current_value,100.0)))
-            {
-              if (current_value != (char *) NULL)
-                current_value=DestroyString(current_value);
-              ResetMaxMemoryRequest();
-              return(SetPolicyValue(domain,name,value));
-            }
-          if (current_value != (char *) NULL)
-            current_value=DestroyString(current_value);
+          ResetMaxMemoryRequest();
+          return(MagickTrue);
         }
       if (LocaleCompare(name,"memory-map") == 0)
         {
           if (LocaleCompare(value,"anonymous") != 0)
             return(MagickFalse);
           ResetVirtualAnonymousMemory();
-          return(SetPolicyValue(domain,name,value));
+          return(MagickTrue);
         }
       if (LocaleCompare(name,"precision") == 0)
         {
           ResetMagickPrecision();
-          return(SetPolicyValue(domain,name,value));
-        }
-      if (LocaleCompare(name,"shred") == 0)
-        {
-          current_value=GetPolicyValue("system:shred");
-          if ((current_value == (char *) NULL) ||
-              (StringToInteger(value) > StringToInteger(current_value)))
-            {
-              if (current_value != (char *) NULL)
-                current_value=DestroyString(current_value);
-              return(SetPolicyValue(domain,name,value));
-            }
-          if (current_value != (char *) NULL)
-            current_value=DestroyString(current_value);
+          return(MagickTrue);
         }
       break;
     }
