@@ -17,7 +17,7 @@
 %                               December 2021                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -187,11 +187,11 @@ static Image *ReadQOIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -431,28 +431,30 @@ ModuleExport void UnregisterQOIImage(void)
 static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
-  QuantumType
-    quantum_type;
-
-  MagickBooleanType
-    status;
-
   const Quantum
     *p;
 
-  size_t
-    channels,
-    colorspace,
-    end,
-    run,
-    idx,
-    i;
+  MagickBooleanType
+    status;
 
   qoi_rgba_t
     px,
     pp,
     lut[64];
 
+  QuantumType
+    quantum_type;
+
+  size_t
+    channels,
+    colorspace,
+    end,
+    run;
+
+  ssize_t
+    idx,
+    i;
+
   /*
     Open output image file.
   */
@@ -460,10 +462,10 @@ static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
@@ -491,8 +493,8 @@ static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
   (void) WriteBlobString(image,"qoif");
   (void) WriteBlobMSBLong(image,(unsigned int) image->columns);
   (void) WriteBlobMSBLong(image,(unsigned int) image->rows);
-  (void) WriteBlobByte(image,(const unsigned char) channels);
-  (void) WriteBlobByte(image,(const unsigned char) colorspace);
+  (void) WriteBlobByte(image,(unsigned char) channels);
+  (void) WriteBlobByte(image,(unsigned char) colorspace);
   /*
     Initialize encoding state.
   */
@@ -509,7 +511,7 @@ static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
     Do the actual encoding.
   */
   end=image->rows * image->columns;
-  for (i=0; i < end; i++) {
+  for (i=0; i < (ssize_t) end; i++) {
     pp=px;
     px.rgba.r=ScaleQuantumToChar(GetPixelRed(image,p));
     px.rgba.g=ScaleQuantumToChar(GetPixelGreen(image,p));
@@ -523,22 +525,20 @@ static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
         run++;
         if (run == 62)
           {
-            (void) WriteBlobByte(image,QOI_OP_RUN |
-              (const unsigned char) (run - 1));
+            (void) WriteBlobByte(image,QOI_OP_RUN | (unsigned char) (run - 1));
             run=0;
           }
         continue;
       }
     if (run > 0)
       {
-        (void) WriteBlobByte(image,QOI_OP_RUN |
-          (const unsigned char) (run - 1));
+        (void) WriteBlobByte(image,QOI_OP_RUN | (unsigned char) (run - 1));
         run=0;
       }
     idx=QOI_COLOR_HASH(px) % 64;
     if (lut[idx].v == px.v)
       {
-        (void) WriteBlobByte(image,QOI_OP_INDEX | (const unsigned char) idx);
+        (void) WriteBlobByte(image,QOI_OP_INDEX | (unsigned char) idx);
         continue;
       }
     lut[QOI_COLOR_HASH(px) % 64]=px;
@@ -593,7 +593,7 @@ static MagickBooleanType WriteQOIImage(const ImageInfo *image_info,Image *image,
       }
   }
   if (run > 0)
-    (void) WriteBlobByte(image,QOI_OP_RUN | (const unsigned char) (run - 1));
+    (void) WriteBlobByte(image,QOI_OP_RUN | (unsigned char) (run - 1));
   /*
     Write the QOI end marker: seven 0x00 bytes followed by 0x01.
   */
